/*
 * Copyright 2011-2012 Arx Libertatis Team (see the AUTHORS file)
 *
 * This file is part of Arx Libertatis.
 *
 * Arx Libertatis is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Arx Libertatis is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Arx Libertatis.  If not, see <http://www.gnu.org/licenses/>.
 */
/* Based on:
===========================================================================
ARX FATALIS GPL Source Code
Copyright (C) 1999-2010 Arkane Studios SA, a ZeniMax Media company.

This file is part of the Arx Fatalis GPL Source Code ('Arx Fatalis Source Code'). 

Arx Fatalis Source Code is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

Arx Fatalis Source Code is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with Arx Fatalis Source Code.  If not, see 
<http://www.gnu.org/licenses/>.

In addition, the Arx Fatalis Source Code is also subject to certain additional terms. You should have received a copy of these 
additional terms immediately following the terms and conditions of the GNU General Public License which accompanied the Arx 
Fatalis Source Code. If not, please request a copy in writing from Arkane Studios at the address below.

If you have questions concerning this license or the applicable additional terms, you may contact in writing Arkane Studios, c/o 
ZeniMax Media Inc., Suite 120, Rockville, Maryland 20850 USA.
===========================================================================
*/
// Code: Cyril Meynier
//
// Copyright (c) 1999-2000 ARKANE Studios SA. All rights reserved

#ifndef ARX_PHYSICS_COLLISIONS_H
#define ARX_PHYSICS_COLLISIONS_H

#include <stddef.h>

#include "graphics/data/Mesh.h"
#include "platform/Flags.h"
#include "math/MathFwd.h"

const size_t MAX_IN_SPHERE = 20;

extern size_t MAX_IN_SPHERE_Pos;
extern short EVERYTHING_IN_SPHERE[MAX_IN_SPHERE + 1];
extern size_t EXCEPTIONS_LIST_Pos;
extern short EXCEPTIONS_LIST[MAX_IN_SPHERE + 1];
extern bool DIRECT_PATH;

bool ARX_COLLISION_Move_Cylinder(IO_PHYSICS * ip, INTERACTIVE_OBJ * io, float MOVE_CYLINDER_STEP, CollisionFlags flags = 0);
float CheckAnythingInCylinder(EERIE_CYLINDER * cyl, INTERACTIVE_OBJ * ioo, long flags = 0);

enum CheckAnythingInSphereFlag {
	 CAS_NO_NPC_COL        = (1<<0),
	 CAS_NO_SAME_GROUP     = (1<<1),
	 CAS_NO_BACKGROUND_COL = (1<<2),
	 CAS_NO_ITEM_COL       = (1<<3),
	 CAS_NO_FIX_COL        = (1<<4),
	 CAS_NO_DEAD_COL       = (1<<5)
};
DECLARE_FLAGS(CheckAnythingInSphereFlag, CASFlags)
DECLARE_FLAGS_OPERATORS(CASFlags)

//except source...
bool CheckAnythingInSphere(EERIE_SPHERE * sphere, long source, CASFlags flags = 0, long * num = NULL);

//except source...
bool CheckEverythingInSphere(EERIE_SPHERE * sphere, long source, long targ = -1);

//except source...
EERIEPOLY * CheckBackgroundInSphere(EERIE_SPHERE * sphere);
 
bool IsCollidingIO(INTERACTIVE_OBJ * io, INTERACTIVE_OBJ * ioo);

/*!
 * @param ignoreNoCollisionFlag true if the IO_NO_COLLISIONS on the interactive object should be ignored
 */
bool CheckIOInSphere(EERIE_SPHERE * sphere, long target, bool ignoreNoCollisionFlag = false);

bool AttemptValidCylinderPos(EERIE_CYLINDER * cyl, INTERACTIVE_OBJ * io, CollisionFlags flags);
bool IO_Visible(Vec3f * orgn, Vec3f * dest, EERIEPOLY * epp, Vec3f * hit);

void ANCHOR_BLOCK_By_IO(INTERACTIVE_OBJ * io, long status);
void ANCHOR_BLOCK_Clear();
float CylinderPlatformCollide(EERIE_CYLINDER * cyl, INTERACTIVE_OBJ * io);
bool IsAnyNPCInPlatform(INTERACTIVE_OBJ * pfrm);
void PushIO_ON_Top(INTERACTIVE_OBJ * ioo, float ydec);

#endif // ARX_PHYSICS_COLLISIONS_H
